<?php

/**
 * Provides autocomplete suggestions using the Searchstax "/emsuggest" endpoint.
 */
class SearchStaxAutocompleteSuggester extends SearchApiAutocompleteSuggesterPluginBase {

  /**
   * {@inheritdoc}
   */
  public static function supportsIndex(SearchApiIndex $index): bool {
    $utility = searchstax_get_utility_service();
    if (!$utility->getConfigEntry('autosuggest_core')) {
      return FALSE;
    }
    try {
      $server = static::getSolrServer($index);
      if (!$server) {
        return FALSE;
      }
      return $utility->isSearchstaxSolr($server->options);
    }
    catch (SearchApiException $ignored) {
      // Ignore, just return FALSE.
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getAutocompleteSuggestions(SearchApiQueryInterface $query, $incomplete_key, $user_input): array {
    $suggestions = [];
    try {
      $server_options = static::getSolrServer($query->getIndex())->options ?? NULL;
      if (!$server_options) {
        return $suggestions;
      }
      $core = searchstax_get_utility_service()->getConfigEntry('autosuggest_core');
      if ($core) {
        $last_slash = strrpos($server_options['path'], '/');
        $server_options['path'] = substr($server_options['path'], 0, $last_slash + 1) . $core;
      }
      $connector = new SearchStaxSolrConnection($server_options);
      $response = $connector->makeServletRequest('emsuggest', ['q' => $user_input]);
      $data = static::objectToArray($response->suggest ?? []);
      // Actual data is nested three levels down (suggest.suggester1[user_input]
      // – but no point in hard-coding that).
      while (count($data) === 1) {
        $tmp = reset($data);
        if (!is_array($tmp)) {
          break;
        }
        $data = $tmp;
      }

      foreach ($data['suggestions'] ?? [] as $suggestion) {
        if (!empty($suggestion['term'])) {
          $term = strip_tags($suggestion['term']);
          if (preg_match('#^(.*)<b>(.*)</b>(.*)$#', $suggestion['term'], $m)) {
            $suggestions[] = [
              'suggestion_prefix' => $m[1],
              'user_input' => $m[2],
              'suggestion_suffix' => $m[3],
              'term' => $term,
            ];
          }
          else {
            $suggestions[] = $term;
          }
        }
      }
    }
    catch (SearchApiException $e) {
      watchdog_exception('searchstax', $e, '%type while fetching autocomplete suggestions: @message in %function (line %line of %file).');
    }
    return $suggestions;
  }

  /**
   * Retrieves the Solr server associated with the given index.
   *
   * @param SearchApiIndex $index
   *   The search index.
   *
   * @return SearchApiServer|null
   *   The search server attached to the index, if it uses Solr as its backend.
   *   Or NULL if the index isn't attached to a Solr server.
   *
   * @throws SearchApiException
   *   Thrown if an error occurred while retrieving the server backend plugin.
   */
  protected static function getSolrServer(SearchApiIndex $index): ?SearchApiServer {
    $server = $index->server();
    if (in_array($server->class, ['search_api_solr_service', 'searchstax'])) {
      return $server;
    }
    return NULL;
  }

  /**
   * Converts an object to an associative array recursively.
   *
   * @param object|array $o
   *   The object or list to convert.
   *
   * @return array
   *   An associative array with the same entries, not containing any objects.
   */
  protected static function objectToArray($o): array {
    $a = is_object($o) ? (array) $o : $o;
    foreach ($a as &$v) {
      if (is_object($v) || is_array($v)) {
        $v = static::objectToArray($v);
      }
    }
    return $a;
  }

}
