<?php

/**
 * Provides an interface for the SearchStax utility service.
 */
interface SearchStaxServiceInterface {

  /**
   * Retrieves the module's configuration.
   *
   * @return array
   *   The module's configuration.
   */
  public function getConfig(): array;

  /**
   * Retrieves a single entry from the module's configuration.
   *
   * @param string $name
   *   The name of the setting.
   *
   * @return mixed|null
   *   The setting's value, if any.
   */
  public function getConfigEntry(string $name);

  /**
   * Determines whether tracking is disabled for the current user.
   *
   * @return bool
   *   TRUE if tracking is disabled for the current user role, FALSE otherwise.
   */
  public function isTrackingDisabled(): bool;

  /**
   * Adds search results URL information to a result set.
   *
   * @param array $result_set
   *   The result set to process, in the same format that is returned by
   *   SearchApiQueryInterface::execute(). Passed by reference.
   * @param SearchApiQueryInterface $query
   *   The executed search query.
   * @param array $build
   *   The build array to which, passed by reference.
   * @param string|null $keys
   *   (optional) The search keys for which to track. Will be extracted from the
   *   query if not passed.
   *
   * @return bool
   *   TRUE if tracking was added to the build array, FALSE otherwise.
   */
  public function addTracking(
    array $result_set,
    SearchApiQueryInterface $query,
    array &$build = [],
    ?string $keys = NULL
  ): bool;

  /**
   * Alters a Search API Solr query.
   *
   * If enabled and appropriate, re-routes all search requests through
   * SearchStudio's /emselect handler.
   *
   * Events were only introduced in the 4.2.x release cycle of the Search API
   * Solr module, so depending on the module version this will be either called
   * by our alter hook implementation or by our event subscriber.
   *
   * @param array $call_args
   *   An associative array containing all three arguments to the
   *   SearchApiSolrConnectionInterface::search() call ("query", "params" and
   *   "method") as references.
   * @param SearchApiQueryInterface $query
   *   The underlying Search API query.
   *
   * @see searchstax_search_api_solr_query_alter()
   */
  public function alterSolrQuery(array &$call_args, SearchApiQueryInterface $query): void;

  /**
   * Determines whether the Solr server uses a SearchStax server.
   *
   * @param array $config
   *   The Solr server's configuration array.
   *
   * @return bool
   *   TRUE if the configuration points to a SearchStax Solr server, FALSE
   *   otherwise.
   */
  public function isSearchstaxSolr(array $config): bool;

  /**
   * Retrieves the fulltext keywords of the given query.
   *
   * @param SearchApiQueryInterface $query
   *   The search query.
   *
   * @return string|null
   *   The fulltext keywords (converted to a string, if necessary), if there are
   *   any and the query should be tracked. NULL otherwise.
   */
  public function getQueryKeys(SearchApiQueryInterface $query): ?string;

}
